@@ -36,6 +36,10 @@ class BaseAudioConfig(Coqpit):
             Enable / Disable sound normalization to reconcile the volume differences among samples. Defaults to False.
         do_trim_silence (bool):
             Enable / Disable trimming silences at the beginning and the end of the audio clip. Defaults to ```True```.
+        do_amp_to_db_linear (bool, optional):
+            enable/disable amplitude to dB conversion of linear spectrograms. Defaults to True.
+        do_amp_to_db_mel (bool, optional):
+            enable/disable amplitude to dB conversion of mel spectrograms. Defaults to True.
         trim_db (int):
             Silence threshold used for silence trimming. Defaults to 45.
         power (float):
@@ -79,7 +83,7 @@ class BaseAudioConfig(Coqpit):
     preemphasis: float = 0.0
     ref_level_db: int = 20
     do_sound_norm: bool = False
-    log_func = "np.log10"
+    log_func: str = "np.log10"
     # silence trimming
     do_trim_silence: bool = True
     trim_db: int = 45
@@ -91,6 +95,8 @@ class BaseAudioConfig(Coqpit):
     mel_fmin: float = 0.0
     mel_fmax: float = None
     spec_gain: int = 20
+    do_amp_to_db_linear: bool = True
+    do_amp_to_db_mel: bool = True
     # normalization params
     signal_norm: bool = True
     min_level_db: int = -100
@@ -182,51 +188,87 @@ class BaseTrainingConfig(Coqpit):
     Args:
         model (str):
             Name of the model that is used in the training.
+
         run_name (str):
             Name of the experiment. This prefixes the output folder name.
+
         run_description (str):
             Short description of the experiment.
+
         epochs (int):
             Number training epochs. Defaults to 10000.
+
         batch_size (int):
             Training batch size.
+
         eval_batch_size (int):
             Validation batch size.
+
         mixed_precision (bool):
             Enable / Disable mixed precision training. It reduces the VRAM use and allows larger batch sizes, however
             it may also cause numerical unstability in some cases.
+
+        scheduler_after_epoch (bool):
+            If true, run the scheduler step after each epoch else run it after each model step.
+
         run_eval (bool):
             Enable / Disable evaluation (validation) run. Defaults to True.
+
         test_delay_epochs (int):
             Number of epochs before starting to use evaluation runs. Initially, models do not generate meaningful
             results, hence waiting for a couple of epochs might save some time.
+
         print_eval (bool):
             Enable / Disable console logging for evalutaion steps. If disabled then it only shows the final values at
             the end of the evaluation. Default to ```False```.
+
         print_step (int):
             Number of steps required to print the next training log.
-        tb_plot_step (int):
+
+        log_dashboard (str): "tensorboard" or "wandb"
+            Set the experiment tracking tool
+
+        plot_step (int):
             Number of steps required to log training on Tensorboard.
-        tb_model_param_stats (bool):
+
+        model_param_stats (bool):
             Enable / Disable logging internal model stats for model diagnostic. It might be useful for model debugging.
             Defaults to ```False```.
+
+        project_name (str):
+            Name of the project. Defaults to config.model
+
+        wandb_entity (str):
+            Name of W&B entity/team. Enables collaboration across a team or org.
+
+        log_model_step (int):
+            Number of steps required to log a checkpoint as W&B artifact
+
         save_step (int):ipt
             Number of steps required to save the next checkpoint.
+
         checkpoint (bool):
             Enable / Disable checkpointing.
+
         keep_all_best (bool):
             Enable / Disable keeping all the saved best models instead of overwriting the previous one. Defaults
             to ```False```.
+
         keep_after (int):
             Number of steps to wait before saving all the best models. In use if ```keep_all_best == True```. Defaults
             to 10000.
+
         num_loader_workers (int):
             Number of workers for training time dataloader.
+
         num_eval_loader_workers (int):
             Number of workers for evaluation time dataloader.
+
         output_path (str):
-            Path for training output folder. The nonexist part of the given path is created automatically.
-            All training outputs are saved there.
+            Path for training output folder, either a local file path or other
+            URLs supported by both fsspec and tensorboardX, e.g. GCS (gs://) or
+            S3 (s3://) paths. The nonexist part of the given path is created
+            automatically. All training artefacts are saved there.
     """
 
     model: str = None
@@ -237,14 +279,19 @@ class BaseTrainingConfig(Coqpit):
     batch_size: int = None
     eval_batch_size: int = None
     mixed_precision: bool = False
+    scheduler_after_epoch: bool = False
     # eval params
     run_eval: bool = True
     test_delay_epochs: int = 0
     print_eval: bool = False
     # logging
+    dashboard_logger: str = "tensorboard"
     print_step: int = 25
-    tb_plot_step: int = 100
-    tb_model_param_stats: bool = False
+    plot_step: int = 100
+    model_param_stats: bool = False
+    project_name: str = None
+    log_model_step: int = None
+    wandb_entity: str = None
     # checkpointing
     save_step: int = 10000
     checkpoint: bool = True
